==> ns-1.0b2.tar.gz: ns-1.0b2/trace.cc <==
/*
 * Copyright (c) 1990-1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: trace.cc,v 1.14 96/03/09 18:41:11 mccanne Exp $ (LBL)";
#endif

#include <stdio.h>
#include <stdlib.h>
#include "trace.h"
#include "packet.h"

/*
 * tcl command interface
 */
class TraceMatcher : public Matcher {
public:
	TraceMatcher() : Matcher("trace") { }
        TclObject* match(const char* id) {
		return (new Trace());
	}
} trace_matcher;

Trace::Trace() : file_(0), callback_(0)
{
}

Trace::~Trace()
{
}

FILE* Trace::getfile(Tcl& tcl, const char* id)
{
	FILE* fp;
	if (Tcl_GetOpenFile(tcl.interp(), (char*)id, 1, 1, &fp) != TCL_OK)
		return (0);
	return (fp);
}

/*
 * $trace detach
 * $trace flush
 * $trace attach $fileID
 * $trace callback $proc
 */
int Trace::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 2) {
		if (strcmp(argv[1], "detach") == 0) {
			file_ = 0;
			return (TCL_OK);
		}
		if (strcmp(argv[1], "flush") == 0) {
			fflush(file_);
			return (TCL_OK);
		}
	} else if (argc == 3) {
		if (strcmp(argv[1], "attach") == 0) {
			file_ = getfile(tcl, argv[2]);
			if (file_ == 0) {
				tcl.resultf("trace: can't attach %s for writing", argv[2]);
				return (TCL_ERROR);
			}
			return (TCL_OK);
		}
		if (strcmp(argv[1], "callback") == 0) {
			const char* cb = argv[2];
			delete callback_;
			if (*cb == 0)
				callback_ = 0;
			else {
				callback_ = new char[strlen(cb) + 1];
				strcpy(callback_, cb);
			}
			return (TCL_OK);
		}
	}
	tcl.resultf("ns trace object: no such command %s", argv[1]);
	return (TCL_ERROR);
}

char* pt_names[] = {
	PT_NAMES
};

void Trace::format(int tt, int s, int d, Packet* p)
{
	const char* name = pt_names[p->type_];
	if (name == 0)
		abort();

	char flags[5];
	flags[0] = (p->flags_ & PF_ECN) ? 'C' : '-';
	flags[1] = (p->flags_ & PF_PRI) ? 'P' : '-';
	flags[2] = (p->flags_ & PF_USR1) ? '1' : '-';
	flags[3] = (p->flags_ & PF_USR2) ? '2' : '-';
	flags[4] = 0;

	sprintf(wrk_, "%c %g %d %d %s %d %s %d %d.%d %d.%d %d %d",
		tt, Scheduler::instance().clock(), s, d,
		name, p->size_, flags, p->class_,
		p->src_, p->sport_, p->dst_, p->dport_,
		p->seqno_, p->uid_);
}

void Trace::dump()
{
	if (file_ != 0) {
		fputs(wrk_, file_);
		putc('\n', file_);
	}
	if (callback_ != 0) {
		Tcl& tcl = Tcl::instance();
		tcl.evalf("%s { %s }", callback_, wrk_);
	}
}

void Trace::hop(int src, int dst, Packet* p)
{
	format('h', src, dst, p);
	dump();
}

void Trace::enque(int src, int dst, Packet* p)
{
	format('+', src, dst, p);
	dump();
}

void Trace::deque(int src, int dst, Packet* p)
{
	format('-', src, dst, p);
	dump();
}

void Trace::drop(int src, int dst, Packet* p)
{
	format('d', src, dst, p);
	dump();
}

==> ns-1.1.tar.gz: ns-1.1/trace.cc <==
/*
 * Copyright (c) 1990-1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: trace.cc,v 1.16 96/04/19 12:52:45 mccanne Exp $ (LBL)";
#endif

#include <stdio.h>
#include <stdlib.h>
#include "trace.h"
#include "packet.h"

/*
 * tcl command interface
 */
class TraceMatcher : public Matcher {
public:
	TraceMatcher() : Matcher("trace") { }
        TclObject* match(const char* id) {
		return (new Trace());
	}
} trace_matcher;

Trace::Trace() : channel_(0), callback_(0)
{
}

Trace::~Trace()
{
}

/*
 * $trace detach
 * $trace flush
 * $trace attach $fileID
 * $trace callback $proc
 */
int Trace::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 2) {
		if (strcmp(argv[1], "detach") == 0) {
			channel_ = 0;
			return (TCL_OK);
		}
		if (strcmp(argv[1], "flush") == 0) {
			Tcl_Flush(channel_);
			return (TCL_OK);
		}
	} else if (argc == 3) {
		if (strcmp(argv[1], "attach") == 0) {
			int mode;
			const char* id = argv[2];
			channel_ = Tcl_GetChannel(tcl.interp(), (char*)id,
						  &mode);
			if (channel_ == 0) {
				tcl.resultf("trace: can't attach %s for writing", id);
				return (TCL_ERROR);
			}
			return (TCL_OK);
		}
		if (strcmp(argv[1], "callback") == 0) {
			const char* cb = argv[2];
			delete callback_;
			if (*cb == 0)
				callback_ = 0;
			else {
				callback_ = new char[strlen(cb) + 1];
				strcpy(callback_, cb);
			}
			return (TCL_OK);
		}
	}
	tcl.resultf("ns trace object: no such command %s", argv[1]);
	return (TCL_ERROR);
}

char* pt_names[] = {
	PT_NAMES
};

void Trace::format(int tt, int s, int d, Packet* p)
{
	const char* name = pt_names[p->type_];
	if (name == 0)
		abort();

	char flags[5];
	flags[0] = (p->flags_ & PF_ECN) ? 'C' : '-';
	flags[1] = (p->flags_ & PF_PRI) ? 'P' : '-';
	flags[2] = (p->flags_ & PF_USR1) ? '1' : '-';
	flags[3] = (p->flags_ & PF_USR2) ? '2' : '-';
	flags[4] = 0;

	sprintf(wrk_, "%c %g %d %d %s %d %s %d %d.%d %d.%d %d %d",
		tt, Scheduler::instance().clock(), s, d,
		name, p->size_, flags, p->class_,
		p->src_, p->sport_, p->dst_, p->dport_,
		p->seqno_, p->uid_);
}

void Trace::dump()
{
	if (channel_ != 0) {
		/*
		 * tack on a newline (temporarily) instead
		 * of doing two writes
		 */
		int n = strlen(wrk_);
		wrk_[n] = '\n';
		wrk_[n + 1] = 0;
		(void)Tcl_Write(channel_, wrk_, n + 1);
		wrk_[n] = 0;
	}
	if (callback_ != 0) {
		Tcl& tcl = Tcl::instance();
		tcl.evalf("%s { %s }", callback_, wrk_);
	}
}

void Trace::hop(int src, int dst, Packet* p)
{
	format('h', src, dst, p);
	dump();
}

void Trace::enque(int src, int dst, Packet* p)
{
	format('+', src, dst, p);
	dump();
}

void Trace::deque(int src, int dst, Packet* p)
{
	format('-', src, dst, p);
	dump();
}

void Trace::drop(int src, int dst, Packet* p)
{
	format('d', src, dst, p);
	dump();
}
